//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Zobrazovanie textu na LCD DEM 16101
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

#define RS       1 << 10            // volba udaje(1)/prikaz(0)
#define RW       1 << 9             // volba citanie(1)/zapis(0)
#define E        1 << 8             // povolenie citanie/zapisu

// Pocet krokov casovaca na 1 us
#define STEPS   ((unsigned int) ((MCK/8) + 500000) / 1000000L)

// Rychlost USART1
#define USART1_BAUD_RATE 115200

//  *****************************************************
//  Mikrosekundove oneskorenie (max. cca 10 000)
//  *****************************************************
void delay(unsigned int us)
{
  AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC0);  // povolenie hodin pre TC0
  AT91C_BASE_TC0->TC_CMR = 1;                      // volba hodin (MCK/8)
  AT91C_BASE_TC0->TC_RC = us * STEPS;              // nastavenie porovnavacieho registra C
  AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN |        // povolenie hodin
                           AT91C_TC_SWTRG;         // restart casovaca

  // cakanie na ukoncenie oneskorenia
  // ked je nastaveny compare C flag, je hotovo
  while ((AT91C_BASE_TC0->TC_SR & AT91C_TC_CPCS) == 0) asm ("nop");
    
  AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_TC0);  // zakazanie hodin pre TC0
}

//  *******************************************************
//  Zapis do LCD
//  typ = 1(zapis udajov)/0(zapis prikazu)
//  *******************************************************
void LCD_write(char data, char typ)
{ 
  // nastavenie RS
  if (typ)
    AT91C_BASE_PIOA->PIO_SODR = RS;   // udaje
   else
    AT91C_BASE_PIOA->PIO_CODR = RS;   // prikaz
  // RW = 0 - zapis
  AT91C_BASE_PIOA->PIO_CODR = RW;
  // udaje
  AT91C_BASE_PIOA->PIO_CODR = 0xff;   // vymazanie predoslych udajov
  AT91C_BASE_PIOA->PIO_SODR = data;   // nastavenie novych udajov
  // E = 1
  AT91C_BASE_PIOA->PIO_SODR = E;
  // cakanie min. 230 ns
  delay(1);
  // strobovanie (E = 0)
  AT91C_BASE_PIOA->PIO_CODR = E;
  // dlhsie cakanie (aspon 43 us)
  delay(45);
}

//  *******************************************************
//  Inicializacia LCD
//  *******************************************************
void LCD_init(void)
{
  // cakanie aspon 30 ms
  delay(35000);
  // nastavenie funkcie
  LCD_write(0x38,0);
  // cakanie aspon 39 us
  delay(40);
  // zapnutie displeja a kurzora
  LCD_write(0x0e,0);
  // cakanie aspon 39 us
  delay(40);
  // vymazanie displeja
  LCD_write(0x01,0);
  // cakanie aspon 1,53 ms
  delay(1600);
  // nastavenie rezimu - posun zapnuty a inkrementovanie
  LCD_write(0x06,0);
}

//  *******************************************************
//  Inicializacia USART1
//  *******************************************************
void USART1_init(void)
{
  // Konfiguracia PIO radica do rezimu periferie USART1
  // PIO ma zakazany pristup na dane piny
  AT91C_BASE_PIOA->PIO_PDR = AT91C_PA21_RXD1 | AT91C_PA22_TXD1;
  // povolenie periferie A
  AT91C_BASE_PIOA->PIO_ASR = AT91C_PA21_RXD1 | AT91C_PA22_TXD1;

  // povolenie hodin pre USART
  AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_US1;

  // konfiguracia USART
  AT91C_BASE_US1->US_IDR = 0xffff;        // zakazanie preruseni
  // reset vysielaca a prijimaca a zakazanie ich funkcie
  AT91C_BASE_US1->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX | AT91C_US_RXDIS | AT91C_US_TXDIS;

  AT91C_BASE_US1->US_BRGR = MCK/(USART1_BAUD_RATE * 16);  // rychlost
  AT91C_BASE_US1->US_MR = AT91C_US_CHRL_8_BITS |    // 8 bitove slovo
                          AT91C_US_NBSTOP_1_BIT |   // 1 stop bit
                          AT91C_US_PAR_NONE;        // ziadna parita

  // povolenie prijmu znakov
  AT91C_BASE_US1->US_CR = AT91C_US_RXEN;
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ char znak;
  int pozicia;

  //******************* Uvodna inicializacia *******************
  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = RS | RW | E | 0xff;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = RS | RW | E | 0xff;

  //******************* Hlavna cast programu *******************
  // vsetko je nulove
  AT91C_BASE_PIOA->PIO_CODR = RS | RW | E | 0xff;

  // inicializacia LCD
  LCD_init();
  LCD_write(0x80,0);      // nastavenie na prvy znak riadku (adresa 0x00)

  // inicializacia USART 
  USART1_init();

  // pozicia znaku na displeji
  pozicia = 0;
  // nekonecna slucka
  while(1)
  { // cakanie na prijem znaku
    while (!(AT91C_BASE_US1->US_CSR & 1));    // kontrola bitu RXRDY
    // nacitanie znaku
    znak = AT91C_BASE_US1->US_RHR; 
    // vypis znaku
    LCD_write(znak,1);
    // uprava pozicie
    pozicia++;
    if (pozicia == 8)
      LCD_write(0xc0,0);      // nastavenie na prvy znak druhej casti displeja (adresa 0x40)
    if (pozicia == 16)
    { pozicia = 0;
      LCD_write(0x80,0);      // nastavenie na prvy znak prvej casti displeja (adresa 0x00)
    }
  }
}
